@@ -8,25 +8,26 @@ output is returned. FOr many activities in cloudmesh this is sufficient.
 import errno
 import glob
 import os
+import platform as os_platform
 import shlex
+import shutil
+import subprocess
 import sys
 import zipfile
 from distutils.spawn import find_executable
+from pathlib import Path
 from pipes import quote
 from sys import platform
-import platform as os_platform
 
+import psutil
+from cloudmesh.common.StopWatch import StopWatch
 from cloudmesh.common.console import Console
 from cloudmesh.common.dotdict import dotdict
 from cloudmesh.common.util import path_expand, readfile
-import subprocess
-import psutil
 
-import stat
-from pathlib import Path
 
-#from functools import wraps
-#def timer(func):
+# from functools import wraps
+# def timer(func):
 #    @wraps(func)
 #    def wrapper(*args,**kwargs):
 #        print(f"{func.__name__!r} begins")
@@ -36,7 +37,7 @@ from pathlib import Path
 #        return result
 #    return wrapper
 
-#def NotImplementedInWindows(f):
+# def NotImplementedInWindows(f):
 #    def new_f(*args):
 #        if sys.platform == "win32":
 #            Console.error("The method {f.__name__} is not implemented in Windows,"
@@ -48,9 +49,10 @@ from pathlib import Path
 def NotImplementedInWindows():
     if sys.platform == "win32":
         Console.error("The method {f.__name__} is not implemented in Windows,"
-                    " please implement, and/or submit an issue.")
+                      " please implement, and/or submit an issue.")
         sys.exit()
 
+
 class Brew(object):
 
     @classmethod
@@ -69,7 +71,6 @@ class Brew(object):
             print(name, "... error")
             Console.error(r)
 
-
     @classmethod
     def version(cls, name):
         r = Shell.brew("ls", "--version", "name")
@@ -202,33 +203,188 @@ class Shell(object):
     #
     # ls = cls.execute('cmd', args...)
 
+    @staticmethod
+    def run_timed(label, command, encoding=None, service=None):
+        """
+        runs teh command and uses the StopWatch to time it
+        :param label: name of the StopWatch
+        :param command: the command to be executed
+        :param encoding: the encoding
+        :param service: a prefix to the stopwatch label
+        :return:
+        """
+        _label = str(label)
+        print(_label, command)
+        StopWatch.start(f"{service} {_label}")
+        result = Shell.run(command, encoding)
+        StopWatch.stop(f"{service} {_label}")
+        return str(result)
+
+    @staticmethod
+    def run(command, exit="; exit 0", encoding='utf-8'):
+        """
+        executes the command and returns the output as string
+        :param command:
+        :param encoding:
+        :return:
+        """
+
+        if sys.platform == "win32":
+            command = f"{command}"
+        else:
+            command = f"{command} {exit}"
+
+        r = subprocess.check_output(command,
+                                    stderr=subprocess.STDOUT,
+                                    shell=True)
+        if encoding is None or encoding == 'utf-8':
+            return str(r, 'utf-8')
+        else:
+            return r
+
+    @staticmethod
+    def run2(command, encoding='utf-8'):
+        """
+        executes the command and returns the output as string. This command also
+        allows execution of 32 bit commands.
+
+        :param command: the program or command to be executed
+        :param encoding: encoding of the output
+        :return:
+        """
+        if platform.lower() == 'win32':
+            import ctypes
+
+            class disable_file_system_redirection:
+                _disable = ctypes.windll.kernel32.Wow64DisableWow64FsRedirection
+                _revert = ctypes.windll.kernel32.Wow64RevertWow64FsRedirection
+
+                def __enter__(self):
+                    self.old_value = ctypes.c_long()
+                    self.success = self._disable(ctypes.byref(self.old_value))
+
+                def __exit__(self, type, value, traceback):
+                    if self.success:
+                        self._revert(self.old_value)
+
+            with disable_file_system_redirection():
+                command = f"{command}"
+                r = subprocess.check_output(command,
+                                            stderr=subprocess.STDOUT,
+                                            shell=True)
+                if encoding is None or encoding == 'utf-8':
+                    return str(r, 'utf-8')
+                else:
+                    return r
+        elif platform.lower() == 'linux' or platform.lower() == 'darwin':
+            command = f"{command}"
+            r = subprocess.check_output(command,
+                                        stderr=subprocess.STDOUT,
+                                        shell=True)
+            if encoding is None or encoding == 'utf-8':
+                return str(r, 'utf-8')
+            else:
+                return r
+
+    @classmethod
+    def execute(cls,
+                cmd,
+                arguments="",
+                shell=False,
+                cwd=None,
+                traceflag=True,
+                witherror=True):
+        """Run Shell command
+
+        :param witherror: if set to False the error will not be printed
+        :param traceflag: if set to true the trace is printed in case of an error
+        :param cwd: the current working directory in which the command is
+        supposed to be executed.
+        :param shell: if set to true the subprocess is called as part of a shell
+        :param cmd: command to run
+        :param arguments: we do not know yet
+        :return:
+        """
+        # print "--------------"
+        result = None
+        terminal = cls.terminal_type()
+        # print cls.command
+        os_command = [cmd]
+        if terminal in ['linux', 'windows']:
+            os_command = [cmd]
+        elif 'cygwin' in terminal:
+            if not cls.command_exists(cmd):
+                print("ERROR: the command could not be found", cmd)
+                return
+            else:
+                os_command = [cls.command[cls.operating_system()][cmd]]
+
+        if isinstance(arguments, list):
+            os_command = os_command + arguments
+        elif isinstance(arguments, tuple):
+            os_command = os_command + list(arguments)
+        elif isinstance(arguments, str):
+            os_command = os_command + arguments.split()
+        else:
+            print("ERROR: Wrong parameter type", type(arguments))
+
+        if cwd is None:
+            cwd = os.getcwd()
+        try:
+            if shell:
+                if platform.lower() == 'win32':
+                    import ctypes
+                    class disable_file_system_redirection:
+                        _disable = ctypes.windll.kernel32.Wow64DisableWow64FsRedirection
+                        _revert = ctypes.windll.kernel32.Wow64RevertWow64FsRedirection
+
+                        def __enter__(self):
+                            self.old_value = ctypes.c_long()
+                            self.success = self._disable(
+                                ctypes.byref(self.old_value))
+
+                        def __exit__(self, type, value, traceback):
+                            if self.success:
+                                self._revert(self.old_value)
+
+                    if len(os_command) == 1:
+                        os_command = os_command[0].split(' ')
+                    with disable_file_system_redirection():
+                        result = subprocess.check_output(os_command,
+                                                         stderr=subprocess.STDOUT,
+                                                         shell=True,
+                                                         cwd=cwd)
+                else:
+                    result = subprocess.check_output(
+                        os_command,
+                        stderr=subprocess.STDOUT,
+                        shell=True,
+                        cwd=cwd)
+            else:
+                result = subprocess.check_output(
+                    os_command,
+                    # shell=True,
+                    stderr=subprocess.STDOUT,
+                    cwd=cwd)
+        except:
+            if witherror:
+                Console.error("problem executing subprocess",
+                              traceflag=traceflag)
+        if result is not None:
+            result = result.strip().decode()
+        return result
+
     @staticmethod
     def rmdir(top, verbose=False):
 
-        # check if top exists
         p = Path(top)
         if not p.exists():
             return
-
-        for root, dirs, files in os.walk(top, topdown=False):
-            for name in files:
-                try:
-                    filename = os.path.join(root, name)
-                    if verbose:
-                        print("remove", filename)
-                    os.chmod(filename, stat.S_IWUSR)
-                    os.remove(filename)
-                except:
-                    pass
-            for name in dirs:
-                try:
-                    os.rmdir(os.path.join(root, name))
-                except:
-                    pass
         try:
-            os.rmdir(top)
-        except:
-            pass
+            shutil.rmtree(path_expand(top))
+        except OSError as e:
+            print(e.strerror)
+            assert False, "Directory {location} could not be deleted"
 
     @staticmethod
     def dot2svg(filename, engine='dot'):
@@ -287,7 +443,6 @@ class Shell(object):
         })
         return data
 
-
     @classmethod
     def get_python(cls):
         """
@@ -302,7 +457,6 @@ class Shell(object):
         pip_version = Shell.pip("--version").split()[1]
         return python_version_s, pip_version
 
-
     @classmethod
     def check_output(cls, *args, **kwargs):
         """Thin wrapper around :func:`subprocess.check_output`
@@ -329,18 +483,26 @@ class Shell(object):
         """
         return cls.execute('ls', args)
 
-    @classmethod
-    # @NotImplementedInWindows
-    def ps(cls, *args):
+    @staticmethod
+    def ps():
         """
-        executes ps with the given arguments
-        :param args: 
-        :return: 
+        using psutil to return the process information pid, name and comdline,
+        cmdline may be a list
+
+        :return: a list of dicts of process information
         """
-        NotImplementedInWindows()
-        # TODO: tasklist in windows
-        # TODO: make tasklist and ps behave similar, e.g. have the same output,
-        return cls.execute('ps', args)
+        found = []
+        for proc in psutil.process_iter():
+            try:
+                pinfo = proc.as_dict(attrs=['pid', 'name', 'cmdline'])
+            except psutil.NoSuchProcess:
+                pass
+            else:
+                found.append(pinfo)
+        if len(pinfo) == 0:
+            return None
+        else:
+            return found
 
     @classmethod
     def bash(cls, *args):
@@ -509,17 +671,13 @@ class Shell(object):
                                                             host=host))
 
     @classmethod
-    # @NotImplementedInWindows
     def pwd(cls, *args):
         """
         executes pwd with the given arguments
         :param args: 
         :return: 
         """
-        NotImplementedInWindows()
-        # TODO: implement possibly useing echo %cd%
-        #       there is also python os.getcwd(), check if this works on windows
-        return cls.execute('pwd', args)
+        return os.getcwd()
 
     @classmethod
     def rackdiag(cls, *args):
@@ -532,15 +690,13 @@ class Shell(object):
 
     @classmethod
     # @NotImplementedInWindows
-    def rm(cls, *args):
+    def rm(cls, location):
         """
         executes rm with the given arguments
         :param args: 
         :return: 
         """
-        NotImplementedInWindows()
-        # TODO: replace this with shutil.rmtree
-        return cls.execute('rm', args)
+        shutil.rmtree(path_expand(location))
 
     @classmethod
     def rsync(cls, *args):
@@ -726,30 +882,51 @@ class Shell(object):
                 result = result + [line]
         return result
 
+    @classmethod
+    def find_lines_from(cls, lines, what):
+        """
+        returns all lines that come after a particular line
+        :param lines:
+        :param what:
+        :return:
+        """
+        result = []
+        found = False
+        for line in lines:
+            found = found or what in line
+            if found:
+                result = result + [line]
+        return result
 
-    # def __init__(cls):
-    #     """
-    #     identifies parameters for the os
-    #     """
-    #     if cls.operating_system() == "windows":
-    #         cls.find_cygwin_executables()
-    #     else:
-    #         pass
-    #         # implement for cmd, for linux we can just pass as it includes everything
-    #
-    # @classmethod
-    # def find_cygwin_executables(cls):
-    #     """
-    #     find the executables in cygwin
-    #     """
-    #     exe_paths = glob.glob(cls.cygwin_path + r'\*.exe')
-    #     # print cls.cygwin_path
-    #     # list all *.exe in  cygwin path, use glob
-    #     for c in exe_paths:
-    #         exe = c.split('\\')
-    #         name = exe[1].split('.')[0]
-    #         # command['windows'][name] = "{:}\{:}.exe".format(cygwin_path, c)
-    #         cls.command['windows'][name] = c
+
+    @classmethod
+    def find_lines_between(cls, lines, what_from, what_to):
+        """
+        returns all lines that come between the markers
+        :param lines:
+        :param what:
+        :return:
+        """
+        select = Shell.find_lines_from(lines, what_from)
+        select = Shell.find_lines_to(select, what_to)
+        return select
+
+    @classmethod
+    def find_lines_to(cls, lines, what):
+        """
+        returns all lines that come before a particular line
+        :param lines:
+        :param what:
+        :return:
+        """
+        result = []
+        found = True
+        for line in lines:
+            if what in line:
+                return result
+            else:
+                result = result + [line]
+        return result
 
     @classmethod
     def terminal_type(cls):
@@ -799,104 +976,15 @@ class Shell(object):
     @staticmethod
     def get_pid(name, service="psutil"):
         pid = None
-
-        if service == "psutil":
-            for proc in psutil.process_iter():
-                if name in proc.name():
-                    pid = proc.pid
-        elif service == 'grep':
-            result = Shell.ps(["-a"])
-            print (result)
-
-
+        for proc in psutil.process_iter():
+            if name in proc.name():
+                pid = proc.pid
         return pid
 
-    @staticmethod
-    def run(command, exit="; exit 0", encoding='utf-8'):
-        """
-        executes the command and returns the output as string
-        :param command:
-        :param encoding:
-        :return:
-        """
-
-        if sys.platform == "win32":
-            command = f"{command}"
-        else:
-            command = f"{command} {exit}"
-
-        r = subprocess.check_output(command,
-                                    stderr=subprocess.STDOUT,
-                                    shell=True)
-        if encoding is None or encoding == 'utf-8':
-            return str(r, 'utf-8')
-        else:
-            return r
-
-    @staticmethod
-    def run2(command, encoding='utf-8'):
-        """
-        executes the command and returns the output as string. This command also
-        allows execution of 32 bit commands.
-
-        :param command: the program or command to be executed
-        :param encoding: encoding of the output
-        :return:
-        """
-        if platform.lower() == 'win32':
-            import ctypes
-
-            class disable_file_system_redirection:
-                _disable = ctypes.windll.kernel32.Wow64DisableWow64FsRedirection
-                _revert = ctypes.windll.kernel32.Wow64RevertWow64FsRedirection
-
-                def __enter__(self):
-                    self.old_value = ctypes.c_long()
-                    self.success = self._disable(ctypes.byref(self.old_value))
-
-                def __exit__(self, type, value, traceback):
-                    if self.success:
-                        self._revert(self.old_value)
-
-            with disable_file_system_redirection():
-                command = f"{command}"
-                r = subprocess.check_output(command,
-                                            stderr=subprocess.STDOUT,
-                                            shell=True)
-                if encoding is None or encoding == 'utf-8':
-                    return str(r, 'utf-8')
-                else:
-                    return r
-        elif platform.lower() == 'linux' or platform.lower() == 'darwin':
-            command = f"{command}"
-            r = subprocess.check_output(command,
-                                        stderr=subprocess.STDOUT,
-                                        shell=True)
-            if encoding is None or encoding == 'utf-8':
-                return str(r, 'utf-8')
-            else:
-                return r
-
     @staticmethod
     def cms(command, encoding='utf-8'):
         return Shell.run("cms " + command, encoding=encoding)
 
-    @staticmethod
-    def run_timed(label, command, encoding=None, service=None):
-        """
-        runs teh command and uses the StopWatch to time it
-        :param label: name of the StopWatch
-        :param command: the command to be executed
-        :param encoding: the encoding
-        :param service: a prefix to the stopwatch label
-        :return:
-        """
-        _label = str(label)
-        print(_label, command)
-        StopWatch.start(f"{service} {_label}")
-        result = Shell.run(command, encoding)
-        StopWatch.stop(f"{service} {_label}")
-        return str(result)
 
     @classmethod
     def check_python(cls):
@@ -912,7 +1000,9 @@ class Shell(object):
 
         if python_version[0] == 2:
 
-            print("You are running an unsupported version of python: {:}".format(python_version_s))
+            print(
+                "You are running an unsupported version of python: {:}".format(
+                    python_version_s))
 
             # python_version_s = '.'.join(v_string)
             # if (python_version[0] == 2) and (python_version[1] >= 7) and (python_version[2] >= 9):
@@ -924,11 +1014,16 @@ class Shell(object):
 
         elif python_version[0] == 3:
 
-            if (python_version[0] == 3) and (python_version[1] >= 7) and (python_version[2] >= 0):
+            if (python_version[0] == 3) and (python_version[1] >= 7) and (
+                python_version[2] >= 0):
 
-                print("You are running a supported version of python: {:}".format(python_version_s))
+                print(
+                    "You are running a supported version of python: {:}".format(
+                        python_version_s))
             else:
-                print("WARNING: You are running an unsupported version of python: {:}".format(python_version_s))
+                print(
+                    "WARNING: You are running an unsupported version of python: {:}".format(
+                        python_version_s))
                 print("         We recommend you update your python")
 
         # pip_version = pip.__version__
@@ -944,92 +1039,6 @@ class Shell(object):
             print("             pip install -U pip\n")
 
 
-    @classmethod
-    def execute(cls,
-                cmd,
-                arguments="",
-                shell=False,
-                cwd=None,
-                traceflag=True,
-                witherror=True):
-        """Run Shell command
-
-        :param witherror: if set to False the error will not be printed
-        :param traceflag: if set to true the trace is printed in case of an error
-        :param cwd: the current working directory in which the command is
-        supposed to be executed.
-        :param shell: if set to true the subprocess is called as part of a shell
-        :param cmd: command to run
-        :param arguments: we do not know yet
-        :return:
-        """
-        # print "--------------"
-        result = None
-        terminal = cls.terminal_type()
-        # print cls.command
-        os_command = [cmd]
-        if terminal in ['linux', 'windows']:
-            os_command = [cmd]
-        elif 'cygwin' in terminal:
-            if not cls.command_exists(cmd):
-                print("ERROR: the command could not be found", cmd)
-                return
-            else:
-                os_command = [cls.command[cls.operating_system()][cmd]]
-
-        if isinstance(arguments, list):
-            os_command = os_command + arguments
-        elif isinstance(arguments, tuple):
-            os_command = os_command + list(arguments)
-        elif isinstance(arguments, str):
-            os_command = os_command + arguments.split()
-        else:
-            print("ERROR: Wrong parameter type", type(arguments))
-
-        if cwd is None:
-            cwd = os.getcwd()
-        try:
-            if shell:
-                if platform.lower() == 'win32':
-                    import ctypes
-                    class disable_file_system_redirection:
-                        _disable = ctypes.windll.kernel32.Wow64DisableWow64FsRedirection
-                        _revert = ctypes.windll.kernel32.Wow64RevertWow64FsRedirection
-
-                        def __enter__(self):
-                            self.old_value = ctypes.c_long()
-                            self.success = self._disable(ctypes.byref(self.old_value))
-
-                        def __exit__(self, type, value, traceback):
-                            if self.success:
-                                self._revert(self.old_value)
-                    if len(os_command) == 1:
-                        os_command = os_command[0].split(' ')
-                    with disable_file_system_redirection():
-                        result = subprocess.check_output(os_command,
-                                                    stderr=subprocess.STDOUT,
-                                                    shell=True,
-                                                    cwd=cwd)
-                else:
-                    result = subprocess.check_output(
-                        os_command,
-                        stderr=subprocess.STDOUT,
-                        shell=True,
-                        cwd=cwd)
-            else:
-                result = subprocess.check_output(
-                    os_command,
-                    # shell=True,
-                    stderr=subprocess.STDOUT,
-                    cwd=cwd)
-        except:
-            if witherror:
-                Console.error("problem executing subprocess",
-                              traceflag=traceflag)
-        if result is not None:
-            result = result.strip().decode()
-        return result
-
     @classmethod
     def mkdir(cls, directory):
         """
@@ -1082,7 +1091,7 @@ class Shell(object):
         elif platform == "windows":
             os.system("notepad " + filename)
         else:
-           raise NotImplementedError("Editor not configured for OS")
+            raise NotImplementedError("Editor not configured for OS")
 
     @classmethod
     # @NotImplementedInWindows
@@ -1127,12 +1136,15 @@ class Shell(object):
                     for line in r.split():
                         if ":" in line:
                             attribute, value = line.split(":", 1)
-                            attribute = attribute.strip().replace(" ", "_").lower()
+                            attribute = attribute.strip().replace(" ",
+                                                                  "_").lower()
                             value = value.strip()
                             result[attribute] = value
-                    result["distribution"] = result["description"].split(" ")[0].lower()
+                    result["distribution"] = result["description"].split(" ")[
+                        0].lower()
                 except:
-                    Console.error(f"lsb_release not found for the platform {machine}")
+                    Console.error(
+                        f"lsb_release not found for the platform {machine}")
                     raise NotImplementedError
         elif machine == 'darwin':
             result["distribution"] = "macos"
@@ -1194,6 +1206,5 @@ def main():
     print(r)
 
 
-
 if __name__ == "__main__":
     main()
